package frc.robot.subsystems;

import java.util.function.DoubleSupplier;

import com.revrobotics.spark.SparkMax;
import com.revrobotics.spark.SparkBase.PersistMode;
import com.revrobotics.spark.SparkBase.ResetMode;
import com.revrobotics.spark.SparkLowLevel.MotorType;

import edu.wpi.first.wpilibj.DigitalInput;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.ManipulatorConstants;

public class Manipulator extends SubsystemBase {
    private SparkMax manipulatorMotor;

    private DigitalInput coralBeamBreak;
    private DigitalInput algaeBeamBreak;

    public Manipulator() {
        manipulatorMotor = new SparkMax(
            ManipulatorConstants.kManipulatorMotorID, 
            MotorType.kBrushless
        );

        manipulatorMotor.configure(
            ManipulatorConstants.motorConfig, 
            ResetMode.kResetSafeParameters, 
            PersistMode.kPersistParameters
        );

        coralBeamBreak = new DigitalInput(ManipulatorConstants.kCoralBeamBreakID);
        algaeBeamBreak = new DigitalInput(ManipulatorConstants.kAlgaeBeamBreakID);
    }

    /**
     * Runs the manipulator at a set speed with the direction based on the coral parameter
     * 
     * @param speed The speed at which the manipulator runs
     * @param coral Is the manipulator manipulating a coral? (True = Coral, False = Algae)
     * @return Runs manipulator at speed
     */
    public Command runManipulator(DoubleSupplier speed, boolean coral) {
        return run(() -> {
            manipulatorMotor.set(coral ? speed.getAsDouble() : speed.getAsDouble() * -1);
        });
    }

    /**
     * Runs the manipulator until either the algae or coral beam break reads true
     * 
     * @param speed The speed at which the manipulator is run
     * @param coral Is the object a coral? (True = Coral, False = Algae)
     * @return Runs the manipulator until an object is present
     */
    public Command runUntilCollected(double speed, boolean coral) {
        return run(() -> {
            manipulatorMotor.set(coral ? speed : speed * -1);
        }).until(() -> coralBeamBreak.get() || algaeBeamBreak.get());
    }
}
